<?php

namespace Drupal\metatag_import_export_csv\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;

/**
 * Defines a form that configures forms module settings.
 */
class MetaTagDownloadForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'metatag_import_export_download';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['enity_type'] = [
      '#title' => $this->t('Enitity'),
      '#type' => 'select',
      '#required' => TRUE,
      '#options' => $this->getEnityTypes(),
      '#default_value' => 'node',
      '#ajax' => [
        'callback' => '::getBundlesList',
        'event' => 'change',
      ],
    ];
    $form['bundles'] = [
      '#title' => $this->t('Bundle'),
      '#type' => 'select',
      '#multiple' => "multiple",
      '#required' => TRUE,
      '#validated' => TRUE,
      '#options' => $this->getBundles('node'),
    ];
    $form['tags'] = [
      '#title' => $this->t('Tags'),
      '#type' => 'select',
      '#required' => TRUE,
      '#multiple' => "multiple",
      '#size' => 30,
      '#options' => $this->getTags(),
    ];
    $form['delimeter'] = [
      '#title' => $this->t('Delimiter'),
      '#type' => 'textfield',
      '#required' => TRUE,
      '#description' => t('Enter the delimiter for CSV.'),
      '#default_value' => ',',
      '#maxlength' => 2,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#required' => TRUE,
      '#value' => $this->t("Download"),
    ];
    return $form;
  }

  /**
   * Returns entity types.
   */
  public function getEnityTypes() {
    $enityTypes = [
      'node' => 'node',
      'taxonomy_term' => 'Taxonomy Term',
    ];
    return $enityTypes;
  }

  /**
   * Ajax callback for entity type field.
   */
  public function getBundlesList($form, FormStateInterface &$form_state) {
    $ajax_response = new AjaxResponse();
    $enityType = $form_state->getValue('enity_type');
    $form['bundles']['#options'] = $this->getBundles($enityType);
    $ajax_response->addCommand(new ReplaceCommand('.form-item-bundles', $form['bundles']));
    return $ajax_response;
  }

  /**
   * Returns bundles.
   */
  public function getBundles($entity) {
    $bundles = [];
    $bundleList = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entity);
    foreach ($bundleList as $keys => $values) {
      $bundles[$keys] = (string) $values['label'];
    }
    return $bundles;
  }

  /**
   * Returns tags supported by metatag module.
   */
  public function getTags() {
    $tagList = [];
    $metatag_manager = \Drupal::service('metatag.manager');
    $tagGroupList = $metatag_manager->sortedGroupsWithTags();
    foreach ($tagGroupList as $keys => $values) {
      $tagList[$keys] = $values['tags'];
      foreach ($values['tags'] as $tkeys => $tvalues) {
        $tagList[$keys][$tkeys] = $tvalues['label'];
      }
    }
    return $tagList;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $enityType = $form_state->getValue('enity_type');
    $bundles = $form_state->getValue('bundles');
    $tags = $form_state->getValue('tags');
    $delimeter = $form_state->getValue('delimeter');
    $this->metatagImportExportCsvDownload($enityType, $bundles, $tags, $delimeter);
  }

  /**
   * Gets list of all entity and  passes to batch.
   */
  public function metatagImportExportCsvDownload($enityType, $bundles, $tags, $delimeter) {
    $query = \Drupal::entityQuery($enityType);
    switch ($enityType) {
      case'taxonomy_term':
        $query->condition('vid', $bundles, 'IN');
        break;

      case 'node':
        $query->condition('type', $bundles, 'IN');
        $query->sort('created', 'ASC');
        break;
    }

    $nidlist = $query->execute();
    $nidlist = array_values($nidlist);
    $total_count = count($nidlist);
    $operations = [];
    for ($i = 0; $i < $total_count; $i++) {
      $operations[] = [
        '\Drupal\metatag_import_export_csv\MetatagExport::generateExportCsv',
        [
          $nidlist[$i],
          $enityType,
          $tags,
          $delimeter,
        ],
      ];
    }
    $batch = [
      'operations' => $operations,
      'finished' => '\Drupal\metatag_import_export_csv\MetatagExport::downloadCsv',
      'title' => $this->t('Metatags Export'),
      'init_message' => $this->t('Export Process is starting.'),
      'progress_message' => $this->t('Processed @current out of @total.'),
      'error_message' => $this->t('Batch  has encountered an error.'),
    ];
    batch_set($batch);
  }

}
