<?php

namespace Drupal\metatag_import_export_csv;

use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Class for batch process for metatag export.
 */
class MetatagExport {

  /**
   * Batch Callback to generate csv.
   */
  public static function generateExportCsv($id, $enityType, $tags, $delimeter, &$context = []) {
    $entity = [];
    $entity = entity_load($enityType, $id);
    if (!empty($entity)) {
      $entityTags = metatag_generate_entity_metatags($entity);
      $tagsList = [];
      $tagsList['entity_id'] = $id;
      switch ($enityType) {
        case'taxonomy_term':
          $tagsList['entity_title'] = $entity->getName();
          $tagsList['entity_bundle'] = $entity->getVocabularyId();
          break;

        case 'node':
          $tagsList['entity_title'] = $entity->getTitle();
          $tagsList['entity_bundle'] = $entity->getType();
          break;
      }
      $tagsList['entity_type'] = $enityType;
      $tagsList['alias'] = \Drupal::service('path.alias_manager')->getAliasByPath('/' . $enityType . '/' . $id);
      foreach ($tags as $keys => $values) {
        if (!empty($entityTags[$keys])) {
          if (isset($entityTags[$keys]['#attributes']['content'])) {
            $tagsList[$keys] = $entityTags[$keys]['#attributes']['content'];
          }
          elseif (isset($entityTags[$keys]['#attributes']['href'])) {
            $tagsList[$keys] = $entityTags[$keys]['#attributes']['href'];
          }
          else {
            $tagsList[$keys] = "";
          }
        }
        else {
          $tagsList[$keys] = "";
        }
      }
      $context['results']['rows'][] = $tagsList;
      $context['results']['delimeter'] = $delimeter;
      $context['results']['tags'] = $tags;
      $context['results']['entity_type'] = $enityType;
    }
  }

  /**
   * Batch 'finished' callback used to download csv.
   */
  public static function downloadCsv($success, $results, $operations) {
    $messenger = \Drupal::messenger();
    if ($success) {
      $delimeter = $results['delimeter'];
      $filePath = "public://metatag_import_export_csv/";
      $fileName = "metatag_import_export_csv__" . $results['entity_type'] . ".csv";
      $fileUrl = $filePath . $fileName;
      $response = new Response();
      $response->headers->set('Content-Type', 'text/csv');
      $response->headers->set('Content-Disposition', 'attachment;filename=' . $fileName);
      $header = $results['tags'];
      array_unshift($header, 'entity_id', 'entity_title', 'entity_bundle', 'entity_type', 'alias');
      $file = file_unmanaged_save_data('', $fileUrl, FILE_EXISTS_REPLACE);
      if (!($file)) {
        $messenger->addMessage(t('Unable to write to file system. Check permissions and try again.'), 'error');
        return;
      }
      $fp = fopen($file, 'w');
      if (!$fp) {
        $messenger->addMessage(t('Unable open file for writing. Check permissions and try again.'), 'error');
        return;
      }
      fputcsv($fp, $header, $delimeter);
      foreach ($results['rows'] as $row) {
        fputcsv($fp, $row, $delimeter);
      }
      fclose($fp);
      $link = Link::fromTextAndUrl(t($fileName), Url::fromUri(file_create_url($fileUrl)));
      $messenger->addMessage(
        t('Download Csv : @args',
              [
                '@args' => $link->toString(),
              ]
        )
      );
    }
    else {
      $error_operation = reset($operations);
      $messenger->addMessage(
        t('An error occurred while processing @operation with arguments : @args',
              [
                '@operation' => $error_operation[0],
                '@args' => print_r($error_operation[0], TRUE),
              ]
        ), 'error'
      );
    }
  }

}
