<?php

namespace Drupal\metatag_import_export_csv;

/**
 * Class for batch process for metatag import.
 */
class MetatagImport {

  /**
   * Batch callback for parsing the csv.
   */
  public static function importCsv($data, $index1, $index2, $headers, &$context = []) {
    $entity = [];
    if (!empty($data[$index1])) {
      $entity = entity_load($data[$index2], $data[$index1]);
      $tags = [];
      foreach ($headers as $keys => $values) {
        if (trim($data[$keys]) == '_blank') {
          $tags[$values] = "";
        }
        else {
          $tags[$values] = $data[$keys];
        }
      }
      // Remove non meta tags fields.
      unset($tags['entity_id'], $tags['entity_title'], $tags['entity_type'], $tags['alias']);
      try {
        $entity->set('field_meta_tags', serialize($tags));
        $entity->save();
        $context['results']['success'][] = $data[$index1];
      }
      catch (\Exception $e) {
        $context['results']['error'][] = $e;
      }
    }
  }

  /**
   * Batch Finished callback for import.
   */
  public static function importFinish($success, $results, $operations) {
    $messenger = \Drupal::messenger();
    if ($success) {
      foreach ($results['success'] as $keys => $values) {
        $messenger->addMessage(
        t('Successfully updated entity: @args',
              [
                '@args' => $values,
              ]
        )
        );
      }
    }
    else {
      $error_operation = reset($operations);
      $messenger->addMessage(
        t('An error occurred while processing @operation with arguments : @args', [
          '@operation' => $error_operation[0],
          '@args' => print_r($error_operation[0], TRUE),
        ]
        ), 'error'
      );
    }
  }

}
